# calculate ABT statistic and the corresponding lambda's
# see Eq 2.8 of Wu et al. 2017, AOAS

ABT <- function(Y, Z, G, PHI.pf)
{
  nind <- length(Y)
  if (nrow(Z) != nind || nrow(G) != nind)
  {
    stop("Dimension of response / covariates in the model is wrong!")
  }
  nind.pf <- nrow(PHI.pf)
  nfam <- nind / nind.pf
  
  PHI.pf.inv <- solve(PHI.pf)
  PHI = kronecker(diag(nfam), PHI.pf)
  PHI.pf.eig <- eigen(PHI.pf)
  PHI.pf.halfinv <- PHI.pf.eig$vectors %*% diag(PHI.pf.eig$values ^ (-0.5)) %*% t(PHI.pf.eig$vectors)
  
  G.unr <- matrix(NA, nind.pf * nfam, nsnp)
  # PinvG <- matrix(NA, nind.pf * nfam, nsnp)
  for (j in 1 : nfam) # calculate G.unr-->DRD.hat and PinvG-->alfreq.hat
  {
    ix.fi <- nind.pf * (j - 1) + (1 : nind.pf)
    G.pf <- G[ix.fi, ]
    G.unr[ix.fi, ] <- PHI.pf.halfinv %*% G.pf
    # PinvG[ix.fi, ] <- PHImix.pf.inv %*% G.pf
  }
  DRD.hat <- cov(G.unr)
  
  VP0 <- calculateV(Y, Z, PHI.pf)
  V <- VP0$V
  SinvP0Sinv <- VP0$SinvP0Sinv # for kernel test
  VTG <- t(V) %*% G # colSums(matrix(rep(V, nsnp), ncol = nsnp) * G)
  VTPHIV <- as.vector(t(V) %*% PHI %*% V)
  # calculate Lambda
  DRD.hat.eig <- eigen(DRD.hat)
  DRD.hat.eigvector <- DRD.hat.eig$vectors
  DRD.hat.eigvalue <- DRD.hat.eig$values
  W <- VTPHIV ^ (-0.5) * DRD.hat.eigvector %*% diag(DRD.hat.eigvalue ^ (-0.5)) %*% t(DRD.hat.eigvector)
  VTGW <- VTG %*% W
  temp <- calculateLambda_Wmat(W, G, SinvP0Sinv)
  Lambda <- temp$Lambda
  # calculate S_ABT
  ABT.tstat <- sum(VTGW ^ 2)
  
  return(list(Lambda = Lambda, tstat = ABT.tstat))
}
