# simulation under the null hypothesis
# for mixed individuals, mixed variants, n = 1600, nsnp = 10, 50, 100, rho = 0.2, 0.5, 0.8, nsimu = 100
library(mvtnorm)

sourcepath <- "C:/Work/ABTpackage/Source"
resultpath <- "C:/Work/ABTpackage/Result"
temppath <- "C:/Work/ABTpackage/Temp"
source(paste(sourcepath, "generatepedstrmix.R", sep = "/"))
source(paste(sourcepath, "generatelatcorrmat.R", sep = "/"))
source(paste(sourcepath, "generateZY_H0.R", sep = "/"))
source(paste(sourcepath, "generatefdrG.R", sep = "/"))
source(paste(sourcepath, "generatepedG.R", sep = "/"))
source(paste(sourcepath, "generateG.R", sep = "/"))
source(paste(sourcepath, "estimateVC.R", sep = "/"))
source(paste(sourcepath, "calculateV.R", sep = "/"))
source(paste(sourcepath, "calculateLambda_Wmat.R", sep = "/"))
source(paste(sourcepath, "ABT.R", sep = "/"))

sd <- 0
nsimu <- 100
nfam <- 80
pedstrmix <- generatepedstrmix()
nind.pf <- nrow(pedstrmix)
b <- c(1, 0.6)
s2 <- c(2, 2)

load(paste(temppath, "PHImix.pf.RData", sep = "/"))

for (nsnp in c(10, 50, 100))
{
  for (eta in c(0.2, 0.5, 0.8))
  {
    S.ABT <- rep(NA, nsimu)
    Lambda.mat <- matrix(NA, nsimu, nsnp)
    
    set.seed(sd)
    latcorrmatforfdr <- generatelatcorrmat(nsnp, eta)
    runt <- system.time({
      for(i in 1 : nsimu)
      {
        # generate phenotype data (for safety reason, generate Z, Y in each simulation)
        ZY <- generateZY_H0(nfam, pedstrmix, b, s2, PHImix.pf)
        Y <- ZY$Y
        Z <- ZY$Z # Z includes intercept
        # generate genotype data
        alfreqforfdr <- c(runif(round(nsnp / 2), 0.1, 0.5), runif(nsnp - round(nsnp / 2), 0.01, 0.1))
        G <- generateG(nfam, pedstrmix, alfreqforfdr, latcorrmatforfdr)
        # calculate ABT statistic and lambda's
        ABT.result <- ABT(Y, Z, G, PHImix.pf)
        Lambda.mat[i, ] <- ABT.result$Lambda
        S.ABT[i] <- ABT.result$tstat
        
      }
    })
    save(nsimu, runt, S.ABT, Lambda.mat, file = paste(resultpath, "/H0_IX_sd_", sd, "_nsnp_", nsnp, "_eta_", eta, ".RData", sep = ""))
  }
}
